package de.elisabetheckstaedt.dipl.common;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.elisabetheckstaedt.dipl.real.AbKM;
import de.elisabetheckstaedt.dipl.real.Stromverbraucher;
import de.elisabetheckstaedt.dipl.real.Verdunstungsrueckkuehler;

/**
 * reprsentiert alle Investitionsgter
 * @author Elisabeth Eckstdt
 *
 */
public abstract class Einzelobjekt extends Kostenstelle{

	/**
	 * [a]
	 * Gibt die Werte des Objekts laut VDI 2067 Blatt 1 Tabelle A3 an. Wird in den Kindklassen gesetzt.
	 */
	protected double Nutzungsdauer;
	/**
	 * % der Investition pro a
	 */
	protected double Instandsetzung;
	/**
	 * % der Investition pro a
	 */
	protected double Wartung;
	
	/**
	 * [h/a]
	 */
	protected double Inspektion;
	/**
	 * [h/a]
	 */
	protected double Bedienung;
	
	protected Einzelobjekt(String name) {
		super(name);
	}

	/**
	 * @param T
	 * @param q
	 * @param paf_invest
	 * = (Erstanschaffung + Ersatzanschaffungskosten - Restwert) * Annuittsfaktor
	 */
	protected void Investitionskosten(double T, double q, double paf_invest) {
		double n = 0; //Ersatzanzahl
		double Ersatzanschaffungskosten = 0;
		double Erstanschaffung = Investition;
		double T_N = Nutzungsdauer;
		double r = paf_invest;
		if (T > T_N)
		{
			n = Math.floor(T / T_N); //gibt eine Ganzzahl raus
			for (int i=0; i<n; i++)
			{
				Ersatzanschaffungskosten = Ersatzanschaffungskosten + Erstanschaffung * Math.pow(r,(i+1)*T_N) /
				Math.pow(q,(i+1)*T_N); 
			}
		}
		double Anschaffungswert = Erstanschaffung * Math.pow(r, T_N * n);
		double Abschreibung = ((n + 1) * T_N - T) / T_N;
		double Abzinsung = 1 / Math.pow(q,T);
		double Restwert = Anschaffungswert * Abschreibung * Abzinsung;
		double a = (q - 1) / (1- Math.pow(q, -T)); //Annuitaetsfaktor
		Investitionskosten = (Erstanschaffung + Ersatzanschaffungskosten - Restwert) * a;
	}

	/**
	 * @param T
	 * @param q
	 * @param paf_invest
	 * = Investition * Instandsetzung/100 * ba (preisdynamischer Annuittsfaktor)
	 */
	protected void Instandsetzungskosten(double T, double q, double paf_invest) {
		double a = (q - 1) / (1- Math.pow(q, -T)); //Annuitaetsfaktor
		double r = paf_invest;
		double b = (1-Math.pow(r/q, T))/(q-r); //Barwertfaktor
		double ba = b*a; //preisdynamischer Annuittsfaktor
		Instandsetzungskosten = Investition * Instandsetzung/100 * ba;
	}

	/**
	 * @param T
	 * @param q
	 * @param Stundenlohn
	 * @param paf_lohn
	 * = Bedienung * Stundenlohn * ba
	 */
	protected void Bedienungskosten(double T, double q, double Stundenlohn, double paf_lohn) {
		double a = (q - 1) / (1- Math.pow(q, -T)); //Annuitaetsfaktor
		double r = paf_lohn;
		double b = (1-Math.pow(r/q, T))/(q-r);
		double ba = b*a;
		Bedienungskosten = Bedienung * Stundenlohn * ba;
	}

	/**
	 * @param T
	 * @param q
	 * @param paf_invest
	 * = Investition * Wartung/100 * ba
	 */
	protected void Wartungskosten(double T, double q, double paf_invest) {
		double a = (q - 1) / (1- Math.pow(q, -T)); //Annuitaetsfaktor
		double r = paf_invest;
		double b = (1-Math.pow(r/q, T))/(q-r);
		double ba = b*a;
		Wartungskosten = Investition * Wartung/100 * ba;
	}
	/**
	 * @param T
	 * @param q
	 * @param Stundenlohn
	 * @param paf_lohn
	 * = Investition * Inspektion * Stundenlohn * ba
	 */
	protected void Inspektionskosten(double T, double q, double Stundenlohn, double paf_lohn) {
		double a = (q - 1) / (1- Math.pow(q, -T)); //Annuitaetsfaktor
		double r = paf_lohn;
		double b = (1-Math.pow(r/q, T))/(q-r);
		double ba = b*a;
		Inspektionskosten = Investition * Inspektion * Stundenlohn * ba;
	}

	/**
	 * schreibt {@link Kostenstelle#Stromkosten} <p>
	 * ruft {@link Kostenstelle#Stromverbrauch}}
	 * @param Strompreis Vektor 8760 lang [/kWh]
	 * @param T Betrachtungszeitraum [a]
	 * @param q Zinssatz [1.05]
	 * @param r Preisaenderungsfaktor [1.027]<p>
	 * = StromkostenJ1 * ba (oder null, wenn kein Stromverbraucher)<p>
	 * (StromkostenJ1 = Stromverbrauch * Strompreis)
	 */
	protected void Stromkosten(double[] Strompreis, double T, double q, double r)
	{
		if (this instanceof Stromverbraucher)
		{
			double StromkostenJ1 = Helper.sum(Helper.Kosten(((Stromverbraucher) this).getStromverbrauch(), Strompreis));
			double a = (q - 1) / (1- Math.pow(q, -T)); //Annuitaetsfaktor
			double b = (1-Math.pow(r/q, T))/(q-r);
			double ba = b*a;
			Stromkosten = StromkostenJ1 * ba;
		}
		else
			Stromkosten = 0;
	}
	
	/**
	 * schreibt {@link Kostenstelle#Wasserkosten} <p>
	 * liest {@link Verdunstungsrueckkuehler#Wasserverbrauch}}
	 * @param Wasserpreis  Vektor 8760 lang [/m]
	 * @param T Betrachtungszeitraum [a]
	 * @param q Zinssatz [1.05]
	 * @param r Preisaenderungsfaktor [1.027]<p>
	 * analog zu Stromkosten
	 */
	protected void Wasserkosten(double[] Wasserpreis, double T, double q, double r)
	{
		if (this instanceof Verdunstungsrueckkuehler)
		{
			double WasserkostenJ1 = Helper.sum(Helper.Kosten(((Verdunstungsrueckkuehler) this).getWasserverbrauch(), Wasserpreis));
			double a = (q - 1) / (1- Math.pow(q, -T)); //Annuitaetsfaktor
			double b = (1-Math.pow(r/q, T))/(q-r);
			double ba = b*a; //TODO Wasserteuerung wird mit Energiepreisnderungsfaktor berechnet
			Wasserkosten = WasserkostenJ1 * ba;
		}
		else
			Wasserkosten = 0;
	}
	
	/**
	 * schreibt {@link Kostenstelle#Waermekosten} <p>
	 * liest {@link AbKM#Waermeverbrauch}}
	 * @param Waermepreis  Vektor 8760 lang [/kWh]
	 * @param T Betrachtungszeitraum [a]
	 * @param q Zinssatz [1.05]
	 * @param r Preisaenderungsfaktor [1.027]
	 * <p> analog zu Stromkosten
	 */
	protected void Waermekosten(double[] Waermepreis, double T, double q, double r)
	{
		if (this instanceof AbKM)
		{
			double WaermekostenJ1 = Helper.sum(Helper.Kosten(((AbKM) this).getWaermeverbrauch(), Waermepreis));
			double a = (q - 1) / (1- Math.pow(q, -T)); //Annuitaetsfaktor
			double b = (1-Math.pow(r/q, T))/(q-r);
			double ba = b*a;
			Waermekosten = WaermekostenJ1 * ba;
		}
		else
			Waermekosten = 0;
	}

	
	/**
	 * fgt bei Einzelobjekten an den Knoten zustzliche Informationen (Name, Nutzungsdauer, Investitionstext) hinzu
	 * @param baum
	 * @param preisdaten
	 * @return Node
	 */
	protected Node KostenXMLout(Document baum, Preisdaten preisdaten)
	{
		Element aktElem = (Element) super.KostenXMLout(baum, preisdaten);
		aktElem.setAttribute("Name", getXML_Name());
		aktElem.setAttribute("Nutzungsdauer", String.valueOf(Nutzungsdauer));
		return aktElem;
	}

	public void setBedienung(double bedienung) {
		Bedienung = bedienung;
	}

	public void setInstandsetzung(double instandsetzung) {
		Instandsetzung = instandsetzung;
	}

	public void setNutzungsdauer(double nutzungsdauer) {
		Nutzungsdauer = nutzungsdauer;
	}

	public void setWartung(double wartung) {
		Wartung = wartung;
	}

	public void setInspektion(double inspektion) {
		Inspektion = inspektion;
	}

	public void setInvestition(double investition) {
		Investition = investition;
	}
}
