package de.elisabetheckstaedt.dipl.common;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Locale;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Hilfsfunktionen zum Aufbau der XML-Dateien, zur formatierten Bildschirmausgabe, Umgang mit Array etc.
 * @author Elisabeth Eckstdt
 */
public class Helper {

	/**
	 * @param array
	 * @param zahl
	 * @return Index der Zahl im Array; -1 wenn zahl nicht enthalten
	 */
	public static int findFirstOcc(double[] array, double zahl)
	{
		ArrayList tmp = new ArrayList(array.length);
		for(int i=0;i<array.length;i++)
			tmp.add(new Double(array[i]));
		return tmp.indexOf(new Double(zahl));
	}
	
	/**
	 * Geradengleichung
	 * @param x
	 * @param x1
	 * @param y1
	 * @param x2
	 * @param y2
	 * @return y
	 */
	public static double ggl_2pkt (double x, double x1, double y1, double x2, double y2)
	{
		return ((x - x1) * (y2-y1) / (x2 - x1) + y1);
	}
	//Arrays
	/**
	 * @param array
	 * @return Minimum des bergebenen Vektors
	 */
	public static double min(double[] array)
	{
		ArrayList tmp = new ArrayList(array.length);
		for(int i=0;i<array.length;i++)
			tmp.add(new Double(array[i]));
		return ((Double)Collections.min(tmp)).doubleValue();
	}
	/**
	 * @param array
	 * @return Maximum des bergebenen Vektors
	 */
	public static double max(double[] array)
	{
		ArrayList tmp = new ArrayList(array.length);
		for(int i=0;i<array.length;i++)
			tmp.add(new Double(array[i]));
		return ((Double)Collections.max(tmp)).doubleValue();
	}
	/**
	 * @param array
	 * @return Summe aller Elemente des bbergebenen Vektors
	 */
	public static double sum(double[] array)
	{
		double sum = 0;
		for(int i=0;i<array.length;i++){
			sum = sum + array[i];
		}
		return sum;
	}
	
	//XML
	/**
	 * hngt den bergebenen Wert (mit Euroformat formatiert) als Knoten in das bergebene XML-Dokument
	 * @param doc
	 * @param Vaterknoten
	 * @param Knotenname
	 * @param Wert
	 * @see Helper#Euroformat(double, int)
	 */
	public static void Wertknoten(Document doc, Element Vaterknoten, String Knotenname, double Wert) 
	{
		Vaterknoten.appendChild(doc.createElement(Knotenname).appendChild(doc.createTextNode(Euroformat(Wert,0))).getParentNode());
	}
	/**
	 * hngt den bergebenen Text als Knoten in das bergebene XML-Dokument
	 * @param doc
	 * @param Vaterknoten
	 * @param Knotenname
	 * @param Inhalt
	 */
	public static void Textknoten(Document doc, Element Vaterknoten, String Knotenname, String Inhalt) 
	{
		Vaterknoten.appendChild(doc.createElement(Knotenname).appendChild(doc.createTextNode(Inhalt)).getParentNode());
	}
	/**
	 * setzt den bergebenen Wert (mit Euroformat formatiert) als Attribut des Vaterknotens
	 * @param Vaterknoten
	 * @param Attributname
	 * @param Wert
	 */
	public static void Wertattribut(Element Vaterknoten, String Attributname, double Wert) 
	{
		Vaterknoten.setAttribute(Attributname, Euroformat(Wert,0));
	}
	//Zahlen formatieren
	/**
	 * ruft Helper.Zahlformat(Zahl, Kommastellenzahl)
	 * @param Zahl
	 * @param Kommastellenzahl
	 * @return String der bergebenen Zahl mit der bestimmten Kommastellenzahl und Eurozeichen dahinter
	 * @see Helper#Zahlformat
	 */
	public static String Euroformat(double Zahl, int Kommastellenzahl)
	{
		String s = Zahlformat(Zahl, Kommastellenzahl) +  " ";
		return s;
	}
	/**
	 * @param Zahl
	 * @param Kommastellenzahl
	 * @return String er bergebenen Zahl mit der bestimmten Kommastellenzahl
	 */
	public static String Zahlformat(double Zahl, int Kommastellenzahl)
	{
		DecimalFormat df = (DecimalFormat)DecimalFormat.getInstance(Locale.GERMAN);
		if (Kommastellenzahl == 2)
		{
		df.applyPattern( "#,###,##0.00" );
		}
		else if (Kommastellenzahl == 0)
		{
			df.applyPattern( "#,###,##0" );
		}
		else if (Kommastellenzahl == 3)
		{
			df.applyPattern( "#,###,##0.000" );
		}
		else if (Kommastellenzahl == 4)
		{
			df.applyPattern( "#,###,##0.0000" );
		}
		else
		{
//			df.applyPattern("#,###,##0");
		}
			String s = df.format( new Double(Zahl));
		return s;
	}
	
	/**
	 * @param Verbrauch 8760
	 * @param Preis 8760
	 * @return Kosten 8760 (Produkt aus beiden Parametern)
	 */
	public static double[] Kosten (double[] Verbrauch, double[] Preis)
	{
		if (Verbrauch.length != Preis.length)
			System.err.println("Der Verbrauchs- und Preisvektor sind nicht gleich lang");
		double[] Kosten = new double[Verbrauch.length];
		for (int i=0; i<Verbrauch.length; i++)
		{
			Kosten[i] = Verbrauch[i] * Preis[i];
		}
		return Kosten;
	}
	
	/**
	 * @param array
	 * @return ein Vektor in dem die Elemente umgekehrt wie im bergebenen Vektor angeordnet sind, Ursprungsvektor beibt unberhrt
	 */
	public static double[] reverse (double[] array)
	{
		double[] array2 = new double[array.length];
		for (int i=0; i<array.length; i++)
		{
			array2[i] = array[array.length-1-i];
		}
		return array2;
	}

	/**
	 * @param array
	 * @return Durchschnittswert des bergebenen Vektor
	 */
	public static double Durchschnitt (double[] array)
	{
		double durchschnitt = 0;
		for (int i=0; i<array.length; i++)
		{
			durchschnitt = durchschnitt + array[i];
		}
		durchschnitt = durchschnitt / array.length;
		return durchschnitt;
	}
	/**
	 * @param n
	 * @return Fakultt der bergebenen Zahl
	 */
	public static int fakul(int n)
	{
		int fakultaet = 1;
		for (int i=1; i<=n; i++)
		{
			fakultaet = fakultaet * i;
		}
		return fakultaet;
	}
	
	/**
	 * @param datei
	 * @return Dateiinhalt als String
	 */
	public static String txttoString(File datei)
	{
		char[] Einlesen = new char[new Integer( Math.round(datei.length())).intValue()];
		try {
			FileReader fr = new FileReader(datei);
			fr.read(Einlesen);
		}
		catch (FileNotFoundException e) {e.printStackTrace();} 
		catch (IOException e) {e.printStackTrace();}
		return new String(Einlesen);
	}
}

