package de.elisabetheckstaedt.dipl.real;

/**
 * abstrakte Oberklasse fr alle Kompressionskltemaschinen
 * @author Elisabeth Eckstdt
 *
 */
public abstract class KKM extends Kaeltemaschine{
	
	/**
	 * Q_0/P [1] bei Nennlast
	 */
	protected double epsilon_K;
	
	/**
	 * reicht durch an Vaterkonstruktor
	 * @param name
	 * @param rk
	 * @param teillastregelung
	 */
	protected KKM(String name, Rueckkuehler rk, String teillastregelung)
	{
		super(name, rk, teillastregelung);
	}
	
	/**
	 * setzt Nennleistung, Faktoren<p>
	 * ermittelt mit Vorlauftemeratur das zeta, ruft setZeta()<p>
	 * ermittelt die Abmessungen<p>
	 * ruft Investition()<p>
	 * @param nennleistung
	 */
	public void init(double nennleistung)
	{
		Nennleistung = nennleistung;
		Nutzungsdauer = 15; //a [VDI 2067:2007.03 Tab. A3 2.3.2.1.1]
		Instandsetzung = 2; //% der Investitionssumme pro a
		Wartung = 1; //% der Investitionssumme pro a
		Bedienung = 1; //h/a
		Hilfsantriebe = 0.0; //in der DIN nicht vorgesehen
		if (Rueckkuehler instanceof Wasserrueckkuehler)
			Wasserfaktor = 0.002;
		else Wasserfaktor = 0;
		Inspektion = 0;
		if (Nennleistung < 800)
		{
			laenge = 33/7000 * Nennleistung + 71/35; //VDI 3803 S. 15
			hoehe = 0.001 * Nennleistung + 7/5;
			breite = 1/675 * Nennleistung + 31/35;;
		}
		else
		{
			laenge = 5.8;
			hoehe = 2 * 0.0001 * Nennleistung + 51/25;
			breite = 1.42 * 0.0001 * Nennleistung + 59/35;
		}
		Investition();
		
	}
	
	/**
	 * setzt Investition entsprechend Formel aus IUTA fr Wasserkhlstze
	 */
	protected void Investition()
	{
		if (Nennleistung<10 || Nennleistung > 10000)
		{
			System.err.println("fr Leistungen unter 10 und ber 10.000 kW ist keine Kostenfunktion hinterlegt. Sie wollten eine KKM mit " + Nennleistung + " kW erzeugen");
		}
		this.Investition = (4732.2487 * Math.pow(Nennleistung,-0.7382) + 109.30) * Nennleistung;
	}
	
	/**
	 * ruft PLV()<p>
	 * setzt Stromverbraucher.Stromverbrauch = aktuelle Klteleistung * (Hilfsantriebsfaktor + 1/(eps * PLV))<p>
	 * @param kaelteleistung
	 * @param aussentemperatur nur einer der beiden Parameter wird zum rufen von PLV() verwendet
	 */
	public void Stromverbrauch(double[] kaelteleistung, double[] aussentemperatur)
	{
		double[] PLV;
		if (Kuehlmedium == "Wasser") PLV = PLV(kaelteleistung);
		else PLV = PLV(aussentemperatur);
		for (int i=0; i<8760; i++)
		{
			Stromverbrauch[i] = kaelteleistung[i] * ( 1/(epsilon_K * PLV[i]) + Hilfsantriebe);
		}
	}
	
	/**
	 * @param t_w_a Khlwasseraustrittstemperatur [C]
	 * @return Teillastfaktor der Rckkhlung in dem Zeitschritt mit dieser Khlwasseraustrittstemperatur
	 */
	protected double f_2i(double t_w_a) {
		double untergrenze;
		double obergrenze;
		double m;
		double n;
		if (Rueckkuehler instanceof Trockenrueckkuehler)
		{
			if (this instanceof Schraubenverdichter)
			{
				untergrenze = 15;
				obergrenze = 50;
				m = -0.0486;
				n = 3.1851;
			}
			else //if (this instanceof Turboverdichter) // oder Kolben- oder Turboverdichter
			{
				untergrenze = 15;
				obergrenze = 50;
				m = -0.0249;
				n = 2.1181;
			}
		}
		else //Verdunstungsrckhler
		{
			untergrenze = 12;
			obergrenze = 40;
			m = -0.0307;
			n = 2.0164;
		}
		if (t_w_a<untergrenze || t_w_a >obergrenze) System.err.println("Fr diese Khlwasseraustrittstemperatur ist kein " +
				"Teillastfaktor hinterlegt (AKM.f_2i(t_W_A)) t_W_A=" + t_w_a);
		double f_2i = m * t_w_a + n;
		return f_2i;
	}
	
	/**
	 * @param Kaelteleistung
	 * @return Rckkhlleistung = Klteleistung * (1+1/epsilon)
	 */
	public double[] Rueckkuehlleistung (double[] Kaelteleistung)
	{
		double[] Rueckkuehlleistung = new double[8760];
		for (int i=0; i<8760; i++)
		{
			Rueckkuehlleistung[i] = Kaelteleistung[i] * (1 + 1 / (epsilon_K));// * f_1i(Kaelteleistung[i])));
		}
		return Rueckkuehlleistung;
	}
	
	/**
	 * ruft RK.setNennleistung(), Investition()
	 * @param kaelteleistung_max
	 */
	public void setNennleistung(double kaelteleistung_max)
	{
		Nennleistung = kaelteleistung_max;
		Rueckkuehler.setNennleistung(Nennleistung * (1+1/epsilon_K));
		Investition();
	}

	/**
	 * ruft RK.setNennelistung()
	 * @param epsilon_K
	 */
	public void setEpsilon_K(double epsilon_K) {
		this.epsilon_K = epsilon_K;
		Rueckkuehler.setNennleistung(Nennleistung * (1+1/epsilon_K));
	}
	
	public String Parametertext()
	{
		return "epsilon_K: " + epsilon_K + " [1] Wasserfaktor: " + Wasserfaktor + " [m/kWh_Klte] Hilfsantriebe: " 
		+ Hilfsantriebe + " [kWh_Strom/kWh_Klte]";
	}

	public double getEpsilon_K() {
		return epsilon_K;
	}
	
}
