package de.elisabetheckstaedt.dipl.real;

import de.elisabetheckstaedt.dipl.common.Helper;

/**
 * reprsentiert luftgekhlte (Kompressions)Kltemaschinen nach DIN
 * @author Elisabeth Eckstdt
 */
public class Kompaktanlage extends KoKM{

	/**
	 * nach DIN V 18599-7 
	 */
	private final String Verdichterart;
	
	/**
	 * ruft Luftrueckkuehler() und LRK.init()
	 * @param name
	 * @param verdichterart Kolben (A/B), Scroll (A/B), Schraube (C)
	 * @param teillastregelung
	 */
	public Kompaktanlage(String name, String verdichterart, String teillastregelung) {
		super(name, new Luftrueckkuehler("RK Kompaktanlage dummy"), teillastregelung);
		Verdichterart = verdichterart;
		Rueckkuehler.init(this);
	}
	
	/**
	 * ruft setNennleistung(), setzt Faktoren<p>
	 * ermittelt EER mit VOrlauftemp
	 * @param nennleistung
	 * @param t_K_A
	 */
	public void init(double nennleistung, double t_K_A)
	{
		setNennleistung(nennleistung);
		Nutzungsdauer = 15; //a [VDI 2067:2007.03 Tab. A3 2.3.2.1.1]
		Instandsetzung = 2; //% der Investitionssumme pro a
		Wartung = 1; //% der Investitionssumme pro a
		Bedienung = 1; //h/a
		Inspektion = 0;
		double epsilon_6;
		double epsilon_14;
		if (Verdichterart == "Schraube")
		{
			 epsilon_6 = 3.0;
			 epsilon_14 = 3.7;
		}
		else
		{
			 epsilon_6 = 2.8;
			 epsilon_14 = 3.5;
		}
		EER = Helper.ggl_2pkt(t_K_A, 6, epsilon_6, 14, epsilon_14);
	}
	
	/**
	 * setzt Investition entsprechend Formel aus IUTA fr Klteanlagen
	 */
	protected void Investition() {
		Investition = (4991.3426 * Math.pow(Nennleistung,-0.6794) + 179.63) * Nennleistung;
	}
	public String getBauteilart()
	{
		return "Kompaktanlage";
	}

}
