package de.elisabetheckstaedt.dipl.real;


import de.elisabetheckstaedt.dipl.common.Helper;
import de.elisabetheckstaedt.dipl.common.Klimadatensatz;

/**
 * abstrakte Vaterklasse fr alle Rckkhler
 * @author Elisabeth Eckstdt
 */
public abstract class Rueckkuehler extends Stromverbraucher
{
	/**
	 * [kW] abzufhrende Wrme
	 */
	protected double Nennleistung;
	
	/**
	 * entspricht q_R,elekr in der DIN<p>
	 * P_Hilfs/P_Nenn<p>
	 * [1]<p>
	 */
	protected double Hilfsantriebe;
	
	protected Rueckkuehler(String name)
		{
			super(name);
		}
	
	/**
	 * setzt Faktoren, Nennleistung aus den Paramtern der bergebenen Maschine<p>
	 * ruft Investition()
	 * @param km
	 */
	public void init (Kaeltemaschine km)
	{
		Nutzungsdauer = 20; //a [VDI 2067:2007.03 Tab. A3 2.3.2.2]
		Instandsetzung = 2; //% der Investitionssumme pro a
		Wartung = 1; //% der Investitionssumme pro a
		Bedienung = 0; //h/a
		Inspektion = 0;
		if (km instanceof AbKM)
		{
			Nennleistung = km.Nennleistung * (1+1/((AbKM) km).getZeta_K());
		}
		else
		{
			Nennleistung = km.Nennleistung * (1+1/((KoKM) km).getEER());
		}
		Investition();
	}

	/**
	 * setzte Werte nach IUTA (Achtung: die sind nur von einer Firma ermittelt)
	 */
	protected abstract void Investition();

	/**
	 * @param Kuehlleistung
	 * @param klimadatensatz
	 */
	public abstract void Stromverbrauch(double[] Kuehlleistung, Klimadatensatz klimadatensatz);

	/**
	 * ruft Investition()
	 * @param nennleistung
	 */
	public void setNennleistung(double nennleistung) {
		Nennleistung = nennleistung;
		Investition();
	}

	public void setHilfsantriebe(double hilfsantriebe) {
		Hilfsantriebe = hilfsantriebe;
	}

	public String Investitionstext()
	{
		return super.Investitionstext() + " Nennleistung: " + Helper.Zahlformat(Nennleistung,0) + " kW";
	}
	public String Parametertext()
	{
		return "Hilfsantriebe: " + Hilfsantriebe + " [kWh_Strom/kWh_Klte]";
	}

	public double getHilfsantriebe() {
		return Hilfsantriebe;
	}

	public double getNennleistung() {
		return Nennleistung;
	}

}
