package de.elisabetheckstaedt.dipl.real;

import de.elisabetheckstaedt.dipl.common.Klimadatensatz;
import de.elisabetheckstaedt.dipl.common.Sammelobjekt;
import de.elisabetheckstaedt.dipl.common.Versorgungsaufgabe;

/**
 * Untergruppierung bestimmter Verbraucher eines Teilsystems, charakterisiert durch ein Rohrnetz und einen Kltetrger 
 * @author Elisabeth Eckstdt
 */
public class Subnetz extends Sammelobjekt{

	/**
	 * reprsentiert alle  enthaltenen Pumpen
	 */
	private final Pumpe Pumpen;
	
	/**
	 * verbindendes Rohrnetz
	 */
	private final Rohrnetz Rohrnetz;
	
	/**
	 * alle Anschlussstellen
	 */
	private final Verbraucher[] verbraucher;
	
	/**
	 * i.d.R. Wasser
	 */
	private final Stoff Kaeltetraeger;
	
	public Subnetz(String Name, Pumpe pumpe, Rohrnetz rohrnetz, Verbraucher[] verbraucher) {
		super(Name);
		Pumpen = pumpe;
		Rohrnetz = rohrnetz;
		this.verbraucher = verbraucher;
		Kaeltetraeger = new Wasser();
	}
	
	/**
	 * Investition = 0, bei nur einer Anschlussstelle<p>
	 * ruft Sammelobjekt.Investition() andernfalls
	 */
	public void Investition()
	{
		if (verbraucher.length == 1)
		{
			verbraucher[0].setInvestition(0);
		}
		super.Investition();
	}

	/**
	 * erzeugt dumm-Versorgungsaufgabe und ruft deeren Volumenstrommethode
	 * @param T_A
	 * @param Vorlauftemperatur
	 * @return Volumenstrom [m/s]
	 */
	public double[] volumenstrom(double[] T_A, double Vorlauftemperatur)
	{
		Versorgungsaufgabe dummy = new Versorgungsaufgabe(verbraucher, new Klimadatensatz(T_A));
		return dummy.Volumenstrom(Kaeltetraeger, Vorlauftemperatur);
	}

	/**
	 * @param vp
	 * @return Druckverlust [Pa] vom Rohrnetz (incl. den Einbauten) und den Verbraucheranschlussstellen (Summe daraus)
	 */
	public double[] Druckverlust(double[] vp)
	{
		double[] dp = Rohrnetz.Druckverlust(vp, Kaeltetraeger.Dichte(10));
		double dpv = 0; //Druckverlust aller Hausanschlsse
		for (int j=0; j<verbraucher.length; j++)
		{
			dpv = dpv + verbraucher[j].getDruckverlust();
		}
		if (verbraucher.length == 1) //kein Druckverlust, wenn nur ein Verbraucher, da dann kein Netz und keine Anschlussstelle vorhanden
			{ dpv = 0; }
		for (int i=0; i<8760; i++)
		{
			dp[i] = dp[i] + dpv;
		}
		return dp;
	}
	
	public Object[] getStruktur()
	{
		Object[] versammelt = new Object[3];
		Rohrnetz[] RN = {Rohrnetz};
		Pumpe[] P = {Pumpen};
		versammelt[0] = verbraucher;
		versammelt[1] = P;
		versammelt[2] = RN;
		return versammelt;
	}
	
	public String getBauteilart()
	{
		return "Subnetz";
	}

	public Stoff getKaeltetraeger() {
		return Kaeltetraeger;
	}
	
	public Pumpe getPumpen() {
		return Pumpen;
	}
	
	public Rohrnetz getRohrnetz() {
		return Rohrnetz;
	}

	public Verbraucher[] getVerbraucher() {
		return verbraucher;
	}

}
