package de.elisabetheckstaedt.dipl.real;

import java.util.Arrays;

/**
 * Verbraucher mit Gebudecharakteristik
 * @author Elisabeth Eckstdt
 */
public class Verbraucher_k extends Verbraucher {

	/**
	 * [m]
	 */
	protected double Nutzflaeche; 
	
	/**
	 * [W/m/K]
	 */
	protected double Waermedurchgangswert; 
	
	/**
	 * [C]
	 */
	protected double Raumsolltemperatur;
	
	/**
	 * [W/m] liegt immer an
	 */
	protected double Grundlast;
	
	/**
	 * [W/m] liegt an, wenn Gebude genutzt (wann das ist, ist im Nutzfaktor verschlsselt)
	 * @see Verbraucher#Nutzfaktor
	 */
	protected double Nutzlast;
	
	/**
	 * 8760x[1] gibt an zu welchem Anteil die vorhandene Kltelast im Gebude zu jeder Stunde des Jahres 
	 * gedeckt werden soll
	 */
	protected double[] Lastfaktor;
	
	/**
	 * 3x24x[1] gibt an zu welchem Anteil die vorhandene Kltelast im Gebude zu jeder Stunde der drei 
	 * Tagestypen (Wochen- Sams- und Sonntag) gedeckt werden soll
	 */
	protected double[][] Lastf;
	
	/**
	 * 8760x[1] gibt an zu welchem Anteil die Nutzlast zu jeder Stunde des Jahres anliegt
	 *  @see Verbraucher#Nutzlast
	 */
	protected double[] Nutzfaktor;
	
	/**
	 * 3x24x[1] gibt an zu welchem Anteil die Nutzlast zu jeder Stunde der drei 
	 * Tagestypen (Wochen- Sams- und Sonntag) anliegt
	 * @see Verbraucher#Nutzlast
	 */
	protected double[][] Nutzf;
	
	/**
	 * Konstruktor<p>
	 * setzt Gebudespezifische Werte
	 */
	public Verbraucher_k(String name, double[] ort, double nutzflaeche,
			double waermedurchgangswert, double raumsolltemperatur,
			double grundlast, double nutzlast, 
			double[][] lastf, double[][] nutzf,
			double vorlauftemperatur) {
		super(name, ort, vorlauftemperatur);
		Nutzflaeche = nutzflaeche;
		Waermedurchgangswert = waermedurchgangswert;
		Raumsolltemperatur = raumsolltemperatur;
		Grundlast = grundlast;
		Nutzlast = nutzlast;
		setLastf(lastf);
		setNutzf(nutzf);

	}
	/**
	 * Konstruktor<p>
	 * nur zu benutzen von Kindklassen
	 */
	protected Verbraucher_k(String name, double[] ort, double nutzflaeche,
			double vorlauftemperatur) {
		super(name, ort, vorlauftemperatur);
		Nutzflaeche = nutzflaeche;

	}
	/**
	 * liest Nutzflaeche, Grundlast
	 * @return 8760x[kW]<p>
	 * @see Verbraucher#Nutzflaeche, Verbraucher#Grundlast
	 */
	private double[] Grundlastsumme()
	{
		double[] Grundlastsumme = new double[8760];
		Arrays.fill(Grundlastsumme, Nutzflaeche * Grundlast / 1000);
		return Grundlastsumme;
	}
	/**
	 * liest Nutzflaeche, Nutzlast, Nutzfaktor
	 * @return 8760x[kW]
	 * @see Verbraucher#Nutzflaeche
	 * @see Verbraucher#Nutzlast
	 * @see Verbraucher#Nutzfaktor
	 */
	private double[] Nutzlastsumme()
	{
		double[] Nutzlastsumme = new double[8760];
		for (int i=0; i<8760; i++)
		{
			Nutzlastsumme[i] = Nutzflaeche * Nutzlast * Nutzfaktor[i] / 1000;
		}
		return Nutzlastsumme;
	}
	/**
	 * liest Nutzflaeche, Raumsolltemperatur, Waermedurchgangswert, Lastfaktor
	 * ruft Grundlastsumme(), Nutzlastsumme()
	 * @param Aussentemperatur in C fr jede Stunde des Jahres
	 * @return Klteleistung in kW fr jede Stunde des Jahres
	 * @see Verbraucher_k#Nutzflaeche
	 * @see Verbraucher_k#Raumsolltemperatur
	 * @see Verbraucher_k#Waermedurchgangswert,
	 * @see Verbraucher_k#Lastfaktor
	 * @see Verbraucher_k#Grundlastsumme()
	 * @see Verbraucher_k#Nutzlastsumme()
	 */
	public double[] Kaelteleistung(double[] Aussentemperatur)
	{
		double[] Kaelteleistung = new double[8760];
		double[] Klimalast = new double[8760];
		double[] Waermeeintrag = new double[8760];
		int Geschosszahl = (int)Math.round(Nutzflaeche / 2000); //angenommene Grundflche ca. 2000 m
		if (Geschosszahl == 0)
		{
			Geschosszahl = 1;
		}
		double Grundflaeche = Nutzflaeche / Geschosszahl;
		double kurzeSeite = Grundflaeche / Math.sqrt(Grundflaeche / 2); //angenommenes Seitenverhltnis 1:2
		double Umfang = (kurzeSeite + 2*kurzeSeite) * 2;
		double Fassadenflaeche = Umfang * Geschosszahl * 3; //angenommene Geschosshhe 3m --> Fassadenflaeche in m
		
		double [] GLS = Grundlastsumme();
		double [] NLS = Nutzlastsumme();
		for (int i=0; i<8760; i++)
		{
			if (Aussentemperatur[i] >= Raumsolltemperatur)
			{
				Klimalast[i]  = Fassadenflaeche * (Aussentemperatur[i] - Raumsolltemperatur) * Waermedurchgangswert; //in W
			}
			else
			{
				Klimalast[i] = 0;
			}	
			Waermeeintrag[i] =  GLS[i] + NLS[i] + Klimalast[i]/1000;
			Kaelteleistung[i] = Waermeeintrag[i] * Lastfaktor[i]; //im Lastfaktor ist verschlsselt wann berhaupt gekhlt werden soll
		}
		return Kaelteleistung;
	}
	public void setNutzflaeche(double nutzflaeche) {
		Nutzflaeche = nutzflaeche;
	}
	public void setWaermedurchgangswert(double waermedurchgangswert) {
		Waermedurchgangswert = waermedurchgangswert;
	}
	public void setRaumsolltemperatur(double raumsolltemperatur) {
		Raumsolltemperatur = raumsolltemperatur;
	}
	public void setGrundlast(double grundlast) {
		Grundlast = grundlast;
	}
	public void setNutzlast(double nutzlast) {
		Nutzlast = nutzlast;
	}
	/**
	 * setzt Lastfaktor 8760x1
	 * @param lastf 3x24
	 */
	public void setLastf(double[][] lastf)
	{
		Lastf = lastf;
		double[] Jahresgang = new double[8760];
		double[][] Wert = lastf;
		for (int m=0; m<24;m++)
		{
			Jahresgang[m] = Wert[2][m]; //ein Sonntag zustzlich, um auf 365 zu kommen und zustzliche Feiertage zu reprsentieren
		}
		for (int i=0; i<52; i++)
		{
			for (int j=0; j<7; j++)
			{
				for (int k=0; k<24;k++)
				{
					int js = i*7*24+j*24+k+24;
					switch (j)
					{
						case 0: case 1: case 2: case 3: case 4:
							Jahresgang[js] = Wert[0][k];
							break;
						case 5:
							Jahresgang[js] = Wert[1][k];
							break;
						case 6:
							Jahresgang[js] = Wert[2][k];
					}
				}
			}
		}
		Lastfaktor = Jahresgang;
	}
	/**
	 * setzt Nutzfaktor 8760x1
	 * @param nutzf 3x24
	 */
	public void setNutzf(double[][] nutzf)
	{
		Nutzf = nutzf;
		double[] Jahresgang = new double[8760];
		double[][] Wert = nutzf;
		for (int m=0; m<24;m++)
		{
			Jahresgang[m] = Wert[2][m]; //ein Sonntag zustzlich, um auf 365 zu kommen und zustzliche Feiertage zu reprsentieren
		}
		for (int i=0; i<52; i++)
		{
			for (int j=0; j<7; j++)
			{
				for (int k=0; k<24;k++)
				{
					int js = i*7*24+j*24+k+24;
					switch (j)
					{
						case 0: case 1: case 2: case 3: case 4:
							Jahresgang[js] = Wert[0][k];
							break;
						case 5:
							Jahresgang[js] = Wert[1][k];
							break;
						case 6:
							Jahresgang[js] = Wert[2][k];
					}
				}
			}
		}
		Nutzfaktor = Jahresgang;
	}
	public String toASCII() {
		String SNF = "";
		String SLF = "";
		for (int i=0; i<3; i++)
		{
			for (int j=0; j<24; j++)
			{
				SNF = SNF + Nutzf[i][j];
				SLF = SLF + Lastf[i][j];
				if (!(i==2 && j==23))
				{
					SNF = SNF + " ";
					SLF = SLF + " ";
				}
			}
		}
		String Speichern = "Name: " + Name + " \n" +
		"Ort: " + Ort[0] + " " + Ort[1] + " \n" +
		"Vorlauftemperatur: " + Vorlauftemperatur + " \n" +
		"Gebaeudeart: " + Verbraucherart() + " \n" +
		"Nutzflaeche: " + Nutzflaeche + " \n" +
		"Waermedurchgangswert: " + Waermedurchgangswert + " \n" +
		"Raumsolltemperatur: " + Raumsolltemperatur + " \n" +
		"Grundlast: " + Grundlast + " \n" +
		"Nutzlast: " + Nutzlast +  " \n" +
		"Nutzf: " + SNF + "\n" +
		"Lastf: " + SLF + "\n";
		return Speichern;
	}
	public String Verbraucherart()
	{
		return "Verbraucher_k";
	}
	public double getNutzflaeche() {
		return Nutzflaeche;
	}

	public double getWaermedurchgangswert() {
		return Waermedurchgangswert;
	}

	public double getRaumsolltemperatur() {
		return Raumsolltemperatur;
	}

	public double getGrundlast() {
		return Grundlast;
	}

	public double getNutzlast() {
		return Nutzlast;
	}

	public double[] getLastfaktor() {
		return Lastfaktor;
	}
//	public void setLastfaktor(double[] lastfaktor) {
//		Lastfaktor = lastfaktor;
//
//	}
	public double[][] getLastf() {
		return Lastf;
	}
	public double[] getNutzfaktor() {
		return Nutzfaktor;
	}
	public double[][] getNutzf() {
		return Nutzf;
	}

	
}
