package de.elisabetheckstaedt.dipl.real;

import de.elisabetheckstaedt.dipl.common.Klimadatensatz;

/**
 * Khlwasser wird durch Verdunstung gekhlt
 * @author Elisabeth Eckstdt
 */
public class Verdunstungsrueckkuehler extends Wasserrueckkuehler{
	
	/**
	 * geschlossen/offen
	 */
	protected final String Typ;
	
	/**
	 * radial/axial
	 */
	protected final String Typ2;
	
	protected double[] Wasserverbrauch = new double[8760];
	
	/**
	 * Konstruktor<p>
	 */
	public Verdunstungsrueckkuehler(String name,String Typ, String Typ2)
	{
		super(name);
		this.Typ = Typ;
		this.Typ2 = Typ2;
	}
	
	/**
	 * setzt Hilfsantriebe entsprechend DIN<p>
	 * ruft WRK.init()
	 * @param km
	 */
	public void init(Kaeltemaschine km)
	{
		super.init(km);
		if (Typ == "geschlossen")
		{
			if (Typ2 == "axial")
				Hilfsantriebe = 0.033;
			else if (Typ2 == "radial")
				Hilfsantriebe = 0.04;
			else
				System.err.println("geschlossene Rckkhler gibt es nur als axial oder radial");
		}
		else if (Typ == "offen")//offener Kreislauf == Khlturm
		{
			if (Typ2 == "radial")
			{
				Hilfsantriebe = 0.021;
			}
			else if (Typ2 == "axial")
			{
				Hilfsantriebe = 0.018;
			}
			else
				System.err.println("Fr diesen Typ eines offenen Rckkhlers sind weder Kosten noch Verbrauchswerte " +
						"hinterlegt (" + Typ2 + ").");
		}
		else
			System.err.println("Fr diesen Typ Rckkhler sind weder Kosten noch Verbrauchswerte hinterlegt " +
					"(" + Typ + ").");
	}

	/**
	 * setzt Kostenstelle.Investition() entsprechend IUTA (Werte in IUTA sind nur von einer Firma ermittelt)
	 */
	protected void Investition() {
		if (Typ.equals("geschlossen"))
		{
			if (Typ2.equals("axial") || Typ2.equals("HD")) //Werte aus IUTA VI.24,26 //TODO gelten eigentlich fr 40/35C und wssrige Lsung mit 34Vol% Ethylenglycol, Lufteintritt 25C
			{
				Investition = 37.419 * Nennleistung + 312.93;			
			}
			else if (Typ2.equals("HS"))
			{
				Investition = 46.169 * Nennleistung + 525.02;			
			}
			else if (Typ2.equals("radial") || Typ2.equals("RD")) //Werte aus IUTA VI.29,31 //TODO gelten eigentlich fr 45/40C und wssrige Lsung mit 34Vol% Ethylenglycol, Lufteinritt 32C
			{
				Investition = 49.307 * Nennleistung + 2053.55;			
			}
			else if (Typ2.equals("RS"))
			{
				Investition = 62.671 * Nennleistung + 1711.43;			
			}
		}
		else if (Typ.equals("offen")) //Khlturm
		{
			Investition = (2348.2 * Math.pow((Nennleistung),-1.0398) + 26.15) * Nennleistung;
		}
		if (Nennleistung == 0)
		{
			Investition = 0;
		}
		
	}

	/**
	 * @param klimadatensatz
	 * @param t_W_E
	 * @param t_W_A
	 * @return Nutzungsfaktor des Rckkhlers
	 */
	public double[] f_R(Klimadatensatz klimadatensatz, double[] t_W_E, double[] t_W_A) {
		double[] f_R = new double[8760];
		double Klammer; //Teillastluftmengenverhltnis
		double l_min = 0.87; //relative Mindestluftmenge bei Nennleistung entsprechend DIN V 18599 S.88
		double eta_V = 0.5; //Abkhlgrad bei Nennleistung
		double c_K = 0.8; //Khlturmkonstante
		double Nenner = (Math.log((1-eta_V/c_K)) * l_min);
		double[] t_F = klimadatensatz.feuchtkugeltemperatur();
		for (int i = 0; i<8760; i++)
		{
			double eta_V_i = (t_W_A[i] - t_W_E[i]) / (t_W_A [i]- t_F[i]);
			System.out.println(eta_V_i);
			if (eta_V_i > c_K)
			{
				System.out.println("Khlturmkonstante zu klein (Verdunstungsrckkhler.f_R())");
				eta_V_i = c_K - 0.1;
			}
			double l_min_i = -0.0162 * t_F[i] + 1.2103;
			Klammer = Math.log(1-eta_V_i/c_K) * l_min_i / Nenner;
			f_R[i] = Math.pow(Klammer, 2);
		}
		return f_R;
	}

	public double[] getWasserverbrauch() {
		return Wasserverbrauch;
	}
	public String getBauteilart()
	{
		return "Verdunstungsrueckkuehler";
	}
}
